@@ -35,6 +35,34 @@ from ubuntutools.sponsor_patch.patch import Patch
 from ubuntutools.sponsor_patch.question import ask_for_manual_fixing
 from ubuntutools.sponsor_patch.source_package import SourcePackage
 
+
+def is_command_available(command, check_sbin=False):
+    "Is command in $PATH?"
+    path = os.environ.get('PATH', '/usr/bin:/bin').split(':')
+    if check_sbin:
+        path += [directory[:-3] + 'sbin'
+                 for directory in path if directory.endswith('/bin')]
+    return any(os.access(os.path.join(directory, command), os.X_OK)
+               for directory in path)
+
+
+def check_dependencies():
+    "Do we have all the commands we need for full functionality?"
+    missing = []
+    for cmd in ('patch', 'bzr', 'quilt', 'dput', 'lintian'):
+        if not is_command_available(cmd):
+            missing.append(cmd)
+    if not is_command_available('bzr-buildpackage'):
+        missing.append('bzr-builddeb')
+    if not any(is_command_available(cmd, check_sbin=True)
+               for cmd in ('pbuilder', 'sbuild', 'cowbuilder')):
+        missing.append('pbuilder/cowbuilder/sbuild')
+
+    if missing:
+        Logger.warn("sponsor-patch requires %s to be installed for full "
+                    "functionality", ', '.join(missing))
+
+
 def get_source_package_name(bug_task):
     package = None
     if bug_task.bug_target_name != "ubuntu":
@@ -152,7 +180,7 @@ def extract_source(dsc_file, verbose=False):
         Logger.error("Extraction of %s failed." % (os.path.basename(dsc_file)))
         sys.exit(1)
 
-def get_open_ubuntu_bug_task(launchpad, bug):
+def get_open_ubuntu_bug_task(launchpad, bug, branch=None):
     """Returns an open Ubuntu bug task for a given Launchpad bug.
 
     The bug task needs to be open (not complete) and target Ubuntu. The user
@@ -161,12 +189,21 @@ def get_open_ubuntu_bug_task(launchpad, bug):
     """
     bug_tasks = [BugTask(x, launchpad) for x in bug.bug_tasks]
     ubuntu_tasks = [x for x in bug_tasks if x.is_ubuntu_task()]
+    if branch:
+        branch = branch.split('/')
+
     if len(ubuntu_tasks) == 0:
         Logger.error("No Ubuntu bug task found on bug #%i." % (bug.id))
         sys.exit(1)
     elif len(ubuntu_tasks) == 1:
         task = ubuntu_tasks[0]
-    if len(ubuntu_tasks) > 1:
+    if len(ubuntu_tasks) > 1 and branch and branch[1] == 'ubuntu':
+        tasks = [task for task in ubuntu_tasks
+                      if task.get_series() == branch[2]
+                         and task.package == branch[3]]
+        assert len(tasks) == 1
+        task = tasks[0]
+    elif len(ubuntu_tasks) > 1:
         task_list = [t.get_short_info() for t in ubuntu_tasks]
         Logger.info("%i Ubuntu tasks exist for bug #%i.\n%s", len(ubuntu_tasks),
                     bug.id, "\n".join(task_list))
@@ -250,7 +287,7 @@ def sponsor_patch(bug_number, build, builder, edit, keyid, lpinstance, update,
     #pylint: enable=E1101
 
     (patch, branch) = get_patch_or_branch(bug)
-    task = get_open_ubuntu_bug_task(launchpad, bug)
+    task = get_open_ubuntu_bug_task(launchpad, bug, branch)
 
     dsc_file = task.download_source()
 
@@ -277,7 +314,8 @@ def sponsor_patch(bug_number, build, builder, edit, keyid, lpinstance, update,
                                                          previous_version)
 
         if successful:
-            if source_package.sync(upload, bug_number, bug.owner.name):
+            series = task.get_debian_source_series()
+            if source_package.sync(upload, series, bug_number, bug.owner.name):
                 return
             else:
                 edit = True
